@@ -1,9 +1,13 @@
 from django.core import validators
+from django.core.exceptions import ImproperlyConfigured
+from django.db.models.fields.files import FieldFile
 from django.forms import boundfield
+from django.forms.fields import FileField
 from django.utils.functional import cached_property
 from django.utils.translation import gettext_lazy as _
 
 from formset.widgets import UploadedFileInput
+from formset.upload import get_file_info
 
 
 class CheckboxInputMixin:
@@ -95,6 +99,12 @@ class BoundField(boundfield.BoundField):
             return self.html_name
         return ''
 
+    def value(self):
+        value = super().value()
+        if isinstance(value, FieldFile):
+            return get_file_info(value)
+        return value
+
     def _get_client_messages(self):
         """
         Extract server validation error messages to be rendered by the client side.
@@ -121,7 +131,10 @@ class BoundField(boundfield.BoundField):
         if getattr(self.field, 'max_length', None) is not None:
             data = {'max_length': self.field.max_length}
             max_length_message = _("Ensure this value has at most %(max_length)s characters.")
-            client_messages['too_long'] = server_messages.get('max_length', max_length_message) % data
+            if isinstance(self.field, FileField):
+                client_messages['too_long'] = max_length_message % data
+            else:
+                client_messages['too_long'] = server_messages.get('max_length', max_length_message) % data
         if getattr(self.field, 'min_length', None) is not None:
             data = {'min_length': self.field.min_length}
             min_length_message = _("Ensure this value has at least %(min_length)s characters.")
@@ -140,7 +153,11 @@ class BoundField(boundfield.BoundField):
             data = {'step_value': step_value}
             client_messages['step_mismatch'] = _("Input value must be a multiple of {step_value}.").format(**data)
         client_messages['bad_input'] = validators.ProhibitNullCharactersValidator.message
-        if isinstance(self.field.widget, UploadedFileInput):
+        if isinstance(self.field, FileField):
+            if not isinstance(self.field.widget, UploadedFileInput):
+                raise ImproperlyConfigured(
+                    f"Field of type {self.field.__class__} must use widget inheriting from {UploadedFileInput}"
+                )
             # abuse built-in client errors for failed file upload messages
             client_messages.update(
                 type_mismatch=_("File upload still in progress."),
